@@ -24,7 +24,7 @@ import os
 import sys
 
 from .models import load_model, quantize, dump_config
-from .transforms import fold_rescaling
+from .models.transforms import fold_rescaling
 
 
 def quantize_model(model_path, quant_config, fold=True, name=False, add_deq=True):
