@@ -33,15 +33,15 @@ def assert_equal(x, y, message=None):
     tf.debugging.assert_equal(x, y, message)
 
 
-def assert_less_equal(x, y, message=None):
-  """Assert the condition `x <= y` holds element-wise.
+def assert_less(x, y, message=None):
+  """Assert the condition `x < y` holds element-wise.
 
   If assertions are enabled, this wraps the corresponding tensorflow assertion.
   Otherwise it does nothing.
 
   Args:
     x (:obj:`tensorflow.Tensor`): a tensor
-    y (:obj:`tensorflow.Tensor`): a tensor of the same dtype as and broadcastable to `x`.
+    y (:obj:`tensorflow.Tensor`): a tensor broadcastable to `x`.
     message (str): A string to prefix to the default message. Defaults to None.
 
   Raises:
@@ -49,7 +49,7 @@ def assert_less_equal(x, y, message=None):
     The check can be performed immediately in eager mode or if `x` and `y` are statically known.
   """
   if assert_enabled():
-    tf.debugging.assert_less_equal(x, y, message)
+    tf.debugging.assert_less(tf.cast(x, tf.float32), tf.cast(y, tf.float32), message)
 
 
 def assert_rank_at_least(x, rank, message=None):
