@@ -161,7 +161,6 @@ class QuantizedAttention(Calibrable, Attention):
             quant_config=softmax_quant_conf, name="quantized_softmax")
         # Extract other quantization parameters
         self.ss_scale_bitwidth = self.quant_config.get("ss_scale_bitwidth", 4)
-        self.score_frac_boost = self.quant_config.get("score_frac_boost", 3)
         self.buffer_bitwidth = self.quant_config.get(
             "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
 
@@ -185,10 +184,8 @@ class QuantizedAttention(Calibrable, Attention):
         query = self.separate_heads(inputs[0])
         key = self.separate_heads(inputs[1])
         value = self.separate_heads(inputs[2])
-        # Promote and increase fractional precision of query to have a
-        # score with better precision
+        # Promote query to avoid saturation
         query = query.promote(self.buffer_bitwidth)
-        query = query << self.score_frac_boost
         # Dot product query and key for each head and pairs of tokens
         score = tf.matmul(query, key, transpose_b=True)
         # Rescale the corresponding score
