@@ -18,7 +18,7 @@ import tensorflow as tf
 import keras
 
 from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
-from .layers import deserialize_quant_object, Calibrable, CalibrableVariable, has_shift_info
+from .layers import deserialize_quant_object, Calibrable, CalibrableVariable
 
 __all__ = ["QuantizedDense"]
 
@@ -42,15 +42,10 @@ class QuantizedDense(Calibrable, keras.layers.Dense):
             "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
         assert self.buffer_bitwidth > 0, "The buffer_bitwidth must be a strictly positive integer."
 
-        # Has shift info evaluated only at init
-        self.has_shift_info = has_shift_info()
-        if self.has_shift_info:
-            # Add objects that will store the shift values.
-            self.shift_matmul = CalibrableVariable()
-            self.shift_output_bias = CalibrableVariable()
-            self.shift_bias_output = CalibrableVariable()
-            if self.out_quantizer:
-                self.shift_output = CalibrableVariable()
+        # Add objects that will store the shift values.
+        self.input_shift = CalibrableVariable()
+        self.prod_shift = CalibrableVariable()
+        self.bias_shift = CalibrableVariable()
 
     def call(self, inputs, training=None):
         # raise an error if the inputs are not FixedPoint or tf.Tensor
@@ -71,8 +66,7 @@ class QuantizedDense(Calibrable, keras.layers.Dense):
         # - align all channels on the same fractional bits
         inputs, shift = inputs.promote(self.buffer_bitwidth).align()
         # - update shift values if calibration is enabled
-        if self.has_shift_info:
-            self.shift_matmul(shift)
+        self.input_shift(shift)
 
         outputs = tf.matmul(inputs, kernel)
 
@@ -83,21 +77,13 @@ class QuantizedDense(Calibrable, keras.layers.Dense):
             bias = bias.promote(self.buffer_bitwidth)
             # Align intermediate outputs and biases before adding them
             outputs, shift = outputs.align(bias)
-            if self.has_shift_info:
-                self.shift_output_bias(shift)
+            self.prod_shift(shift)
             bias, shift = bias.align(outputs)
-            if self.has_shift_info:
-                self.shift_bias_output(shift)
+            self.bias_shift(shift)
             outputs = tf.add(outputs, bias)
 
         if self.out_quantizer is not None:
-            internal_out_frac_bits = outputs.frac_bits
             outputs = self.out_quantizer(outputs)
-            out_frac_bits = tf.cast(self.out_quantizer.frac_bits, tf.float32)
-            # update shift values
-            shift = out_frac_bits - internal_out_frac_bits
-            if self.has_shift_info:
-                self.shift_output(shift)
         return outputs
 
     def get_config(self):
