@@ -64,7 +64,8 @@ class QuantizedDepthwiseConv2D(Calibrable, DepthwiseConv2D):
             # inputs that are allowed
             inputs = FixedPoint.quantize(inputs, 0, 8)
 
-        inputs = inputs.promote(self.buffer_bitwidth)
+        # Promote inputs to avoid saturation, then align them
+        inputs, _ = inputs.promote(self.buffer_bitwidth).align()
 
         outputs = backend.depthwise_conv2d(
             inputs,
