@@ -24,12 +24,6 @@ from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
 __all__ = ["Add", "QuantizedAdd", "deserialize_quant_object"]
 
 
-def has_shift_info():
-    if "STORE_SHIFT_INFO" not in os.environ:
-        return False
-    return os.environ["STORE_SHIFT_INFO"] == "1"
-
-
 def deserialize_quant_object(quant_config, object_name, mandatory=True):
     """Wrapper function of tf.keras.utils.deserialize_keras_object.
     It allows to select the right config from the config file dict,
@@ -219,14 +213,9 @@ class QuantizedAdd(Calibrable, Add):
             "buffer_bitwidth", MAX_BUFFER_BITWIDTH) - 1
         self.out_quantizer = deserialize_quant_object(
             self.quant_config, "output_quantizer", False)
-        # Has shift info evaluated only at init
-        self.has_shift_info = has_shift_info()
-        if self.has_shift_info:
-            # Add objects that will store the shift values.
-            self.shift_add_a = CalibrableVariable()
-            self.shift_add_b = CalibrableVariable()
-            if self.out_quantizer:
-                self.shift_output = CalibrableVariable()
+        # Add objects that will store the shift values.
+        self.a_shift = CalibrableVariable()
+        self.b_shift = CalibrableVariable()
 
     def call(self, inputs, training=None):
         a, b = inputs
@@ -245,21 +234,14 @@ class QuantizedAdd(Calibrable, Add):
 
         outputs = tf.add(a, b)
         if self.average:
-            outputs //= 2
+            outputs = outputs >> 1
 
         # Compute shifts
-        if self.has_shift_info:
-            self.shift_add_a(shift_ab)
-            self.shift_add_b(shift_ba)
+        self.a_shift(shift_ab)
+        self.b_shift(shift_ba)
 
         if self.out_quantizer is not None:
-            internal_out_frac_bits = outputs.frac_bits
             outputs = self.out_quantizer(outputs, training=training)
-            if self.has_shift_info:
-                # Update shift values
-                out_frac_bits = tf.cast(self.out_quantizer.frac_bits, tf.float32)
-                shift = out_frac_bits - internal_out_frac_bits
-                self.shift_output(shift)
         return outputs
 
     def get_config(self):
