@@ -42,13 +42,31 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
     .. math::
         LMN(x) = \gamma(x - mean(x))/mad(x) + \beta
 
+    Also, to develop a hardware compatible quantization, we make the approximation of mean
+    on a Power Of Two (PoT):
+
+    .. math::
+        mean(x) = sum(x) * 2**-nb_channels
+
+    Where :math:`nb_channels = round(log2(inputs.shape[axis]))`
+
     .. note::
         A tuning step in the switching procedure between the LN to LMN layer will be required
         to find the :math:`(\gamma, \beta)` parameters that match the standard deviation changes.
     """
 
+    def build(self, input_shape):
+        super().build(input_shape)
+
+        # Save an approximation of nb_channels as PoT to calculate the mean of inputs as:
+        # mean_inputs = sum(inputs) * 2 ** -mean_shift
+        if len(self.axis) > 1:
+            raise ValueError(f"{self.__class__.__name__} does not support multiple axis.")
+        nb_channels = tf.cast(input_shape[self.axis[0]], tf.float32)
+        self.mean_shift = tf.math.round(tf.math.log(nb_channels) / tf.math.log(2.))
+
     def _moments(self, inputs, axes, *args, name=None, **kwargs):
-        """Compute the mean and mean absolute deviation (mad) power of two.
+        """Compute the mean and mean absolute deviation (mad).
 
         Args:
             inputs (:obj:`tensorflow.Tensor`): input tensor.
@@ -59,19 +77,23 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
 
         Returns:
             :obj:`tensorflow.Tensor`, :obj:`tensorflow.Tensor`: mean around the axis and
-                mean absolute deviation (mad) power of two around the axis.
+                mean absolute deviation (mad).
         """
-        # Compute the mean and mad
+        # Compute the mean and mad, taking into account the approximation:
+        # nb_channels \approx 2 ** round(log2(inputs.shape[axis]))
+        nb_channels_inv = 2.0 ** -self.mean_shift
         with ops.name_scope(name, "_moments", [inputs, axes]):
-            mean = tf.math.reduce_mean(
+            mean = tf.math.reduce_sum(
                 inputs, axes, *args, **kwargs, name="mean")
-            mad = tf.math.reduce_mean(tf.math.abs(
+            mean = mean * nb_channels_inv
+            mad = tf.math.reduce_sum(tf.math.abs(
                 inputs - mean), axes, *args, **kwargs, name="mad")
-        return mean, mad ** 2
+            mad = mad * nb_channels_inv
+        return mean, mad
 
     def call(self, inputs, training=None):
         """Use the same call as the parent, changing tf.nn.moments instead of self._moments."""
-        # Compute the axis along which to reduce the mean / variance
+        # Compute the axis along which to reduce the mean / mad
         input_shape = inputs.shape
         ndims = len(input_shape)
 
@@ -92,17 +114,11 @@ class LayerMadNormalization(keras.layers.LayerNormalization):
             inputs = tf.cast(inputs, 'float32')
 
         # Calculate the moments on the last axis (layer activations).
-        mean, variance = self._moments(inputs, self.axis, keepdims=True)
+        mean, mad = self._moments(inputs, self.axis, keepdims=True)
         scale, offset = _broadcast(self.gamma), _broadcast(self.beta)
 
-        # Compute layer normalization using the batch_normalization function.
-        outputs = tf.nn.batch_normalization(
-            inputs,
-            mean,
-            variance,
-            offset=offset,
-            scale=scale,
-            variance_epsilon=self.epsilon)
+        # Compute layer normalization as layer mad normalization
+        outputs = scale * (inputs - mean) / (mad + self.epsilon) + offset
         outputs = tf.cast(outputs, input_dtype)
 
         # If some components of the shape got lost due to adjustments, fix that.
@@ -149,14 +165,13 @@ class QuantizedLayerNormalization(Calibrable, LayerMadNormalization):
             raise TypeError(f"QuantizedLayerNormalization only accepts FixedPoint\
                               inputs. Receives {type(inputs)} inputs.")
 
-        nb_channels = inputs.shape[-1]
-
-        # Increase the value_bits of inputs
-        inputs = inputs.promote(self.buffer_bitwidth)
+        # Increase the value_bits of inputs to avoid saturation, then align them
+        inputs, _ = inputs.promote(self.buffer_bitwidth).align()
 
         # Compute the Mean of the inputs
         sum_x = tf.math.reduce_sum(inputs, self.axis, keepdims=True)
-        mean_x = sum_x // nb_channels
+        # Approximate mean_x = sum_x / 2**mean_shift
+        mean_x = sum_x >> self.mean_shift
 
         # Compute the Absolute Deviation (MAD), aligned inputs and mean before
         inputs, _ = inputs.align(mean_x)
@@ -164,7 +179,8 @@ class QuantizedLayerNormalization(Calibrable, LayerMadNormalization):
         x_shifted = inputs - mean_x
         mad_sum = tf.math.reduce_sum(
             x_shifted.abs(), self.axis, keepdims=True, name="mad")
-        mad = mad_sum // nb_channels
+        # Approximate mad = mad_sum / 2**mean_shift
+        mad = mad_sum >> self.mean_shift
 
         mad = self.mad_quantizer(mad, training=training)
 
