@@ -48,7 +48,7 @@ class QuantizedGlobalAveragePooling2D(Calibrable, GlobalAveragePooling2D):
         super().__init__(**kwargs)
         self.spatial_size_inv = None
         self.quant_config = quant_config
-        self.out_quantizer = deserialize_quant_object(self.quant_config, "output_quantizer")
+        self.out_quantizer = deserialize_quant_object(self.quant_config, "output_quantizer", False)
 
     def build(self, input_shape):
         super().build(input_shape)
@@ -63,11 +63,15 @@ class QuantizedGlobalAveragePooling2D(Calibrable, GlobalAveragePooling2D):
             raise TypeError(f"QuantizedMaxPool2D only accepts FixedPoint inputs. \
                              Receives {type(inputs)} inputs.")
 
-        # Promote input to prevent overflow in the reduce_sum op
-        inputs = inputs.promote(MAX_BUFFER_BITWIDTH)
+        # Promote input to prevent overflow in the reduce_sum op, then align
+        # them
+        inputs, _ = inputs.promote(MAX_BUFFER_BITWIDTH).align()
         inputs_sum = tf.reduce_sum(inputs, axis=[1, 2], keepdims=self.keepdims)
         inputs_mean = inputs_sum * self.spatial_size_inv
-        return self.out_quantizer(inputs_mean, training=training)
+
+        if self.out_quantizer is not None:
+            return self.out_quantizer(inputs_mean, training=training)
+        return inputs_mean
 
     def get_config(self):
         config = super().get_config()
