@@ -83,7 +83,7 @@ class QuantizedSeparableConv2D(Calibrable, SeparableConv2D):
             # inputs that are allowed
             inputs = FixedPoint.quantize(inputs, 0, 8)
 
-        inputs = inputs.promote(self.buffer_bitwidth)
+        inputs, _ = inputs.promote(self.buffer_bitwidth).align()
         dw_outputs = backend.depthwise_conv2d(
             inputs,
             depthwise_kernel,
@@ -92,7 +92,7 @@ class QuantizedSeparableConv2D(Calibrable, SeparableConv2D):
             dilation_rate=self.dilation_rate,
             data_format=self.data_format)
         dw_outputs_q = self.intermediate_quantizer(dw_outputs, training)
-        dw_outputs_q = dw_outputs_q.promote(self.buffer_bitwidth)
+        dw_outputs_q, _ = dw_outputs_q.promote(self.buffer_bitwidth).align()
         outputs = tf.nn.convolution(
             dw_outputs_q,
             pointwise_kernel,
