@@ -23,7 +23,8 @@ from ..tensors import FixedPoint, MAX_BUFFER_BITWIDTH
 
 # Restrict list of exported symbols on default import
 __all__ = ["ClassToken", "QuantizedClassToken",
-           "AddPositionEmbs", "QuantizedAddPositionEmbs"]
+           "AddPositionEmbs", "QuantizedAddPositionEmbs",
+           "ExtractToken", "QuantizedExtractToken"]
 
 
 @tf.keras.utils.register_keras_serializable()
@@ -137,7 +138,12 @@ class QuantizedAddPositionEmbs(Calibrable, AddPositionEmbs):
                               inputs. Receives {type(inputs)} inputs.")
 
         inputs = inputs.promote(self.buffer_bitwidth)
-        outputs = inputs + self.pe_quantizer(self.pe, training=training)
+        pe = self.pe_quantizer(self.pe, training=training).promote(self.buffer_bitwidth)
+
+        # Align intermediate outputs and biases before adding them
+        inputs, _ = inputs.align(pe)
+        pe, _ = pe.align(inputs)
+        outputs = inputs + pe
 
         if self.out_quantizer is not None:
             outputs = self.out_quantizer(outputs)
@@ -147,3 +153,38 @@ class QuantizedAddPositionEmbs(Calibrable, AddPositionEmbs):
         config = super().get_config()
         config["quant_config"] = self.quant_config
         return config
+
+
+@tf.keras.utils.register_keras_serializable()
+class ExtractToken(keras.layers.Layer):
+    """ Wrapper class of `tf.gather` operation that allows to extract a Token.
+
+    Args:
+        token (int): the indice of the token to extract.
+
+    """
+
+    def __init__(self, *args, token, **kwargs):
+        super().__init__(*args, **kwargs)
+        self.token = token
+
+    def call(self, inputs, training=False):
+        return tf.gather(inputs, self.token, axis=1)
+
+    def get_config(self):
+        config = super().get_config()
+        config.update({"token": self.token})
+        return config
+
+
+@tf.keras.utils.register_keras_serializable()
+class QuantizedExtractToken(ExtractToken):
+    """ Quantized version of the ExtractToken layer. Accepts only FixedPoint inputs.
+    """
+
+    def call(self, inputs, training=False):
+        if not isinstance(inputs, FixedPoint):
+            # If the inputs is not a FixedPoint, raise an error
+            raise TypeError(f"QuantizedExtractToken only accepts FixedPoint\
+                              inputs. Receives {type(inputs)} inputs.")
+        return tf.gather(inputs, self.token, axis=1)
