@@ -21,6 +21,7 @@ import warnings
 from keras import Model
 
 from .utils import deep_clone_model, insert_layer
+from .transforms import invert_batchnorm_pooling, fold_batchnorms
 from ..layers import (QuantizedDense, QuantizedLayerNormalization, QuantizedAddPositionEmbs,
                       QuantizedClassToken, QuantizedGELU, QuantizedReLU, QuantizedAdd, QDropout,
                       QuantizedReshape, QuantizedAttention, QuantizedConv2D, QuantizedMaxPool2D,
@@ -37,7 +38,7 @@ CUSTOM_QLAYERS = {
     "Attention": QuantizedAttention, 'MaxPooling2D': QuantizedMaxPool2D,
     'GlobalAveragePooling2D': QuantizedGlobalAveragePooling2D,
     "ExtractToken": QuantizedExtractToken, "SeparableConv2D": QuantizedSeparableConv2D,
-    "Flatten": QuantizedFlatten, "Reciprocal": QuantizedReciprocal,
+    "Flatten": QuantizedFlatten, "Reciprocal": QuantizedReciprocal, "PaddedConv2D": QuantizedConv2D
 }
 # List of Quantizer layer's that do not have a float layer representation
 NO_FLOAT_CUSTOM_QLAYERS = [Dequantizer, FixedPointQuantizer, UniformQuantizer]
@@ -248,7 +249,14 @@ def quantize(model, q_config, add_dequantizer=True):
         # If no match, return the qlayer with config unchanged
         return get_quantize_layer(layer)
 
+    # Invert BN <-> Pooling layers and fold BN into their preceding layers
+    model = invert_batchnorm_pooling(model)
+    model = fold_batchnorms(model)
+
+    # Check if the model has not quantizable layers and add a Dequantizer before
     model = _handle_not_quantizable_layers(model)
+
+    # Quantize the model replacing layers with their quantized version
     new_model = deep_clone_model(model, clone_function=_replace_layer)
     out = new_model.outputs
 
