@@ -26,8 +26,7 @@ from quantizeml.layers.quantizers import Quantizer, Dequantizer
 
 from ..layers import Calibrable
 
-__all__ = ['load_model', 'deep_clone_model', 'set_calibrate',
-           'insert_layer', 'add_shift_info_to_model']
+__all__ = ['load_model', 'deep_clone_model', 'set_calibrate', 'insert_layer']
 
 
 def load_model(model_path, custom_layers={}, compile_model=True):
@@ -173,26 +172,3 @@ def insert_layer(model, target_layer_name, new_layer):
     new_model = Model(inputs=clone_model.inputs, outputs=model_outputs)
     _deep_copy_variables(model, new_model)
     return new_model
-
-
-def add_shift_info_to_model(src_model_path, dst_model_path):
-    """Adds shift information support to a model.
-
-    Args:
-        src_model_path (str): path of the model to update
-        dst_model_path (str): path of the model to write
-    """
-    # Make sure the src model is loaded without shift info
-    os.environ["STORE_SHIFT_INFO"] = "0"
-    print(f"Loading {src_model_path}... ", end="")
-    model = load_model(src_model_path)
-    print("Done")
-    print("Cloning with shift info support... ", end="")
-    # when setting STORE_SHIFT_INFO to 1, the model is cloned with shift info
-    # support
-    os.environ["STORE_SHIFT_INFO"] = "1"
-    new_model = deep_clone_model(model)
-    print("Done")
-    print("Saving... ", end="")
-    new_model.save(dst_model_path)
-    print("Done")
