@@ -17,7 +17,7 @@
 import tensorflow as tf
 
 from .qtensor import QTensor, round_through, floor_through
-from ..debugging import assert_equal, assert_less_equal
+from ..debugging import assert_equal, assert_less
 
 class FixedPoint(QTensor):
     """A Tensor of integer values representing fixed-point numbers
@@ -36,8 +36,6 @@ class FixedPoint(QTensor):
             self.frac_bits = tf.cast(frac_bits, tf.float32)
         else:
             self.frac_bits = tf.convert_to_tensor(frac_bits, tf.float32)
-        assert_less_equal(self.frac_bits, tf.convert_to_tensor(value_bits, tf.float32),
-            "Fractional bits cannot exceed value bits")
         self.value_bits = value_bits
         # We store integer values in a float tensor to speed up calculations
         if isinstance(values, tf.Tensor):
@@ -136,6 +134,8 @@ class FixedPoint(QTensor):
                 raise ValueError("Other must be a FixedPoint")
             max_frac_bits = tf.math.maximum(max_frac_bits, tf.reduce_max(other.frac_bits))
         shift = max_frac_bits - self.frac_bits
+        assert_less(shift, self.value_bits,
+            f"Cannot align the FixedPoint as it saturates its {self.value_bits}-bit buffer.")
         values = FixedPoint._lshift(self.values, shift)
         return FixedPoint(values, max_frac_bits, self.value_bits), shift
 
@@ -161,6 +161,8 @@ class FixedPoint(QTensor):
         frac_bits = tf.cast(frac_bits, tf.float32)
         # Evaluate the shift to apply to reach the target precision
         shift = frac_bits - self.frac_bits
+        assert_less(tf.abs(shift), self.value_bits,
+            f"Cannot downscale the FixedPoint as it saturates its {self.value_bits}-bit buffer.")
         # The shift can be positive (left-shift) or negative (right-shift)
         # In float it can be simulated by flooring after multiplying by the
         # positive/negative shift PoT:
@@ -168,7 +170,7 @@ class FixedPoint(QTensor):
         # - for negative shift, flooring simulates a right_shift.
         values = floor_through(self.values * FixedPoint._pow2(shift))
         # return a new FixedPoint with the target precision and bitwidth
-        return FixedPoint(values, frac_bits, value_bits)
+        return FixedPoint(values, frac_bits, value_bits), shift
 
     @staticmethod
     def _rshift(values, shift):
@@ -193,23 +195,44 @@ class FixedPoint(QTensor):
         return FixedPoint(values, self.frac_bits, self.value_bits)
 
     def __rshift__(self, shift):
+        """Right shift the FixedPoint values
+
+        This operation has no direct equivalent in float arithmetics: it corresponds to a division
+        of the corresponding float by a power-of-two, then a flooring to the quantization interval.
+
+        Args:
+            shift (int): the power of 2 to divide by
+
+        Returns:
+            :obj:`FixedPoint`: the result as a FixedPoint
+        """
         # Convert to a float32 tensor to be able to subtract shift from frac_bits
         shift = tf.convert_to_tensor(shift, tf.float32)
-        # Decrease the fractional bits by shift
-        s_frac_bits = tf.math.maximum(0.0, self.frac_bits - shift)
-        # Divide by 2^shift to decrease precision
-        s_values = FixedPoint._rshift(self.values, shift)
+        # Apply shift simply by increasing the frac bits
+        s_frac_bits = self.frac_bits + shift
         # Return a new FixedPoint with updated fractional bits
-        return FixedPoint(s_values, s_frac_bits, self.value_bits)
+        return FixedPoint(self.values, s_frac_bits, self.value_bits)
 
     def __lshift__(self, shift):
+        """Left shift the FixedPoint values
+
+        This operation has no direct equivalent in float arithmetics: it corresponds to a multiplication
+        of the corresponding float by a power-of-two, then a flooring to the quantization interval.
+
+        Args:
+            shift (int): the power of 2 to multiply by
+
+        Returns:
+            :obj:`FixedPoint`: the result as a FixedPoint
+        """
         # Convert to a float32 tensor to be able to add shift to frac_bits
         shift = tf.convert_to_tensor(shift, tf.float32)
-        # Increase the fractional bits by shift
-        s_frac_bits = self.frac_bits + shift
-        # Multiply by 2^shift to increase precision
-        s_values = FixedPoint._lshift(self.values, shift)
-        # Return a new FixedPoint with updated fractional bits
+        # Try first to absorb the shift by decreasing the frac_bits
+        s_frac_bits = tf.math.maximum(0, self.frac_bits - shift)
+        virtual_shift = self.frac_bits - s_frac_bits
+        # Then apply the remainder on the values
+        s_values = FixedPoint._lshift(self.values, (shift - virtual_shift))
+        # Return a new FixedPoint with updated values and fractional bits
         return FixedPoint(s_values, s_frac_bits, self.value_bits)
 
     def _align_values(self, other):
@@ -324,26 +347,20 @@ class FixedPoint(QTensor):
         return s_values <= o_values
 
     def abs(self):
-        """Returns the absolute value of the QTensor
+        """Returns the absolute value of the FixedPoint
 
         Returns:
-            :obj:`QTensor`: the absolute value.
+            :obj:`FixedPoint`: the absolute value.
         """
         return FixedPoint(tf.math.abs(self.values), self.frac_bits, self.value_bits)
 
-    def __floordiv__(self, other):
-        if isinstance(other, int):
-            return self // FixedPoint(other)
-        elif isinstance(other, FixedPoint):
-            # Align the dividend fractional bits on the max of both terms
-            max_frac_bits = tf.math.maximum(self.frac_bits, other.frac_bits)
-            s_values = self._lshift(
-                self.values, (max_frac_bits - self.frac_bits))
-            # Divide
-            d_values = floor_through(s_values / other.values)
-            # The result fractional bits are reduced by the divisor
-            d_frac_bits = max_frac_bits - other.frac_bits
-            # Return a new FixedPoint
-            return FixedPoint(d_values, d_frac_bits, self.value_bits)
-        raise TypeError(
-            f"Unsupported operand type(s) for //: 'FixedPoint' and '{type(other)}'")
+    def floor(self):
+        """Floors the FixedPoint
+
+        Returns:
+            tuple(:obj:`FixedPoint`, :obj:`tf.Tensor`): a new FixedPoint without
+                fractional bits and the shift that was applied.
+        """
+        # Divide values to remove fractional bits
+        values = FixedPoint._rshift(self.values, self.frac_bits)
+        return FixedPoint(values, 0, self.value_bits), self.frac_bits
