@@ -71,10 +71,16 @@ class QFloat(QTensor):
         scales_res = FixedPoint._pow2(-scales_bits)
         scales = tf.math.maximum(scales, scales_res)
         # Quantize the scales to a FixedPoint
-        scales_int_bits = ceil_through(tf.experimental.numpy.log2(scales))
+        scales_int_bits = tf.math.maximum(0., ceil_through(tf.experimental.numpy.log2(scales)))
         scales_frac_bits = tf.clip_by_value(scales_bits - scales_int_bits,
                                             0, scales_bits)
-        fp_scales = FixedPoint.quantize(scales, scales_frac_bits, scales_bits)
+        # We do not apply the default FixedPoint quantization algorithm, as it
+        # uses a round operation, and we want to make sure the quantized scales
+        # are at least equal to the target scales. Should they be lower, this
+        # would lead to a reduction of the QFloat quantization range and values
+        # near float_max would be clipped.
+        scales_values = ceil_through(scales * FixedPoint._pow2(scales_frac_bits))
+        fp_scales = FixedPoint(scales_values, scales_frac_bits, scales_bits)
         # Quantize the inputs using the approximated FixedPoint scales
         values = round_through(x / fp_scales.to_float())
         return QFloat(values, fp_scales, value_bits)
